#!/usr/bin/env python
# encoding: utf-8
"""
addressbook.py

Python wrapper around Mac OSX Address Book.

from: https://raw.github.com/prabhu/macutils/master/addressbook.py
"""

from AddressBook import *

# List of properties to be ignored.
IGNORED_PROPS = ["com.apple.ABPersonMeProperty", "ABPersonFlags",
                 "com.apple.ABImageData",
                 "com.apple.ABGroupMembersProperty",
                 ]

def getPeopleInAddressBook(group_name=None):
    """
    Returns the people found in the AddressBook.

    optional parameter(group_name = the name of the group)
    """
    ab = ABAddressBook.sharedAddressBook()
    people = None
    if not group_name:
        people = ab.people()
    else:
        for group in ab.groups():
            if group.name() == group_name:
                people = group.members()
    if people == None:
        print "No contacts could be found for given group"
    return _clist(people)


def _clist(slist):
    """
    Method to convert NSArray to python list
    """
    retList = []
    if slist == None:
        return retList
    for p in slist:
        aobj = {}
        for prop in p.allProperties():
            if prop in IGNORED_PROPS:
                continue
            tmpval = p.valueForProperty_(prop)
            if type(tmpval) == ABMultiValueCoreDataWrapper:
                aval = [(_getVal(tmpval.labelAtIndex_(i)),
                         _getVal(tmpval.valueAtIndex_(i)))
                    for i in range(0, tmpval.count())]
            else:
                aval = _getVal(tmpval)
            if aval is not None:
                aobj[prop.lower()] = aval
        retList.append(aobj)
    return retList

def _getVal(tmpval):
    """
    Extract value from unicode, Date or Dictionary object.
    """
    aval = None
    if type(tmpval) == objc.pyobjc_unicode:
        aval = tmpval.lstrip('_$!<').rstrip('>!$_')
    elif issubclass(tmpval.__class__, NSDate):
        aval = tmpval.description()
    elif type(tmpval) == NSCFDictionary:
        aval = dict([(k.lower(), tmpval[k]) for k in tmpval.keys()])
    return aval



from optparse import OptionParser
import sys
import spa_phone


def main():
    parser = OptionParser()
    parser.add_option("-p", "--provider", dest="provider", default="contacts",
        help="Contact provider to use: contacts | outlook. default=contacts")
    parser.add_option("-g", "--group", dest="group",
        help="OS X Contacts group name to sync")

    (options, args) = parser.parse_args()

    if len(args) == 0:
        print 'usage: spa-sync IP-ADDRESS [--provider PROVIDER] [--group GROUP]'
        print 'Please provide the SPA Phone IP address as first argument'
        sys.exit(2)

    if options.provider == 'contacts':
        from spa_sync.providers.contacts import export
    elif options.provider == 'outlook':
        from spa_sync.providers.outlook import export
    else:
        raise ValueError('Valid providers are: contacts and outlook')

    try:
        contacts = export(options.group)
    except Exception as e:
        print 'Could not get contacts from provider (%s)' % e
    else:
        spa_phone.write(args[0], contacts)
        print 'Synced', len(contacts), 'entries'



from spa_sync.providers.outlook import client


def export(group=None):
    """
    Export the contacts for this provider in the correct format
    """
    contacts = []
    for contact in client.get_contacts(group):
        name = contact['FullName'] or contact['CompanyName']
        for key, value in contact['Phones'].items():
            if len(contact['Phones']) > 1:
                    type = key.replace("TelephoneNumber", "")
                    name += ' (%s)' % type
            contacts.append((name, value))
    return contacts



import sys
import win32com.client

if win32com.client.gencache.is_readonly == True:
    # allow gencache to create the cached wrapper objects
    # http://www.py2exe.org/index.cgi/IncludingTypelibs#Solution3
    win32com.client.gencache.is_readonly = False

PHONE_FIELDS = ['BusinessTelephoneNumber',
                'HomeTelephoneNumber',
                'MobileTelephoneNumber']
                 #'AssistantTelephoneNumber',
                 #'OtherTelephoneNumber',
                 #'PrimaryTelephoneNumber',
NAME_FIELDS = ['FullName',
               'CompanyName']


def get_contacts(group=None):
    try:
        outlook_app =\
            win32com.client.gencache.EnsureDispatch("Outlook.Application")
        mapi = outlook_app.GetNamespace("MAPI")
        contacts =\
            mapi.GetDefaultFolder(win32com.client.constants.olFolderContacts)
    except:
        raise ValueError("Outlook could not be found")
    records = []
    for idx in range(1, len(contacts.Items)):
        contact = contacts.Items.Item(idx)
        if contact.Class == win32com.client.constants.olContact:
            record = {}
            for key in NAME_FIELDS:
                record[key] = getattr(contact, key)
                record['Phones'] = {}
                for pkey in PHONE_FIELDS:
                    value = getattr(contact, pkey)
                    if value:
                        record['Phones'][pkey] = value
            categories = getattr(contact, 'Categories').split(', ')
            if not group or group in categories:
                records.append(record)
    return records



import sys

for version in ['2.7', '2.6', '2.5']:
    sys.path.append('/System/Library/Frameworks/Python.framework/Versions/'
                    + version + '/Extras/lib/python/PyObjC')

from spa_sync.providers.contacts import addressbook

__all__ = ['export', ]

def export(group=None):
    """
    Export the contacts for this provider in the correct format
    """
    contacts = []
    for contact in addressbook.getPeopleInAddressBook(group_name=group):
        num_phones = 0
        for type, phone in contact.get('phone', []):
            if not 'fax' in type.lower():
                num_phones += 1
        for type, phone in contact.get('phone', []):
            if 'fax' in type.lower():
                continue
            name = ' '.join([contact.get('first', ''),
                             contact.get('middle', ''),
                             contact.get('last', '')]).strip()
            if name == '':
                name = contact.get('organization', '')
            if num_phones > 1:
                name += ' (%s)' % type
            contacts.append((name, phone))
    return contacts



import sys
from distutils.core import setup
from setuptools import find_packages

if sys.platform == 'win32':
    import py2exe
    extra_options = dict(
        setup_requires=['py2exe'],
        options=dict(
            py2exe=dict(
                excludes=['spa_sync.providers.contacts',],
                compressed=2,
                bundle_files=1,
            ),
        ),
        console=[
            dict(
                script='main.py',
                dest_base='spa_sync',
                description='Command prompt utility to sync SPA phones',
            ),
        ],
        zipfile=None,
    )
else:
    extra_options = dict()

setup(
    name='spa-phone-api',
    version='0.3.0',
    author='Bouke Haarsma',
    author_email='bouke@webatoom.nl',
    packages=find_packages(),
    url='http://github.com/Bouke/spa-phone-api',
    description='Provides an API for Linksys/Sipura VoIP SPA Phones',
    license='MIT',
    long_description=open('README.rst').read(),
    install_requires=[
        'mechanize == 0.2.5',
    ],
    entry_points={
        'console_scripts': [
            'spa-sync = spa_sync:main',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Communications :: Internet Phone',
    ],
    **extra_options
)
